var peer = null;

class RTCPeer {
	constructor(id, isCaller){
		var self = this;
		this.id = id;
		this.isCaller = isCaller;

		//========================================
		//リスト6「ORTCでメディアを送受信する」（一部）ここから
		this.iceOptions = {
			gatherPolicy: 'all',
			iceServers: []
		};
		this.iceGatherer = new RTCIceGatherer(this.iceOptions);
		this.iceTransport = new RTCIceTransport(this.iceGatherer);
		this.dtlsTransport = new RTCDtlsTransport(this.iceTransport);
		//========================================
		//リスト6「ORTCでメディアを送受信する」（一部）ここまで
		this.localParams = null;
		this.remoteParams = null;
		this.localCaps = null;
		this.remoteCaps = null;

		//自分の通信経路候補を探索
		this.iceGatherer.onlocalcandidate = (e) => {
			socket.emit('icecandidate', self.id, e.candidate);
			if(Object.keys(e.candidate).length == 0){
				self.localParams = {
					ice: self.iceGatherer.getLocalParameters(),
					dtls: self.dtlsTransport.getLocalParameters()
				};
				socket.emit('localparams', self.id, self.localParams);
				self.checkParams();
			}
		};

		this.iceTransport.onicestatechange = (e) => {
			console.log('RTCIceTransport.onicestatechange', e.target.state);
			//切断時の処理
			if(e.target.state == 'disconnected'){
				ringOff();
			}
		};
		this.dtlsTransport.ondtlsstatechange = (e) => {
			console.log('RTCDtlsTransport.ondtlsstatechange', e.target.state);
		};

		//========================================
		//リスト6「ORTCでメディアを送受信する」（送信・受信）ここから
		//送信
		//==========
		//RTCRtpSender…映像・音声を送信するオブジェクト
		if(localMediaStream.getVideoTracks().length){
			let vTrack = localMediaStream.getVideoTracks()[0];
			this.videoSender = new RTCRtpSender(vTrack, this.dtlsTransport);
		}
		if(localMediaStream.getAudioTracks().length){
			let aTrack = localMediaStream.getAudioTracks()[0];
			this.audioSender = new RTCRtpSender(aTrack, this.dtlsTransport);
		}

		//受信
		//==========
		//RTCRtpReceiver…映像・音声を受信するオブジェクト
		this.audioReceiver = new RTCRtpReceiver(this.dtlsTransport, 'audio');
		this.videoReceiver = new RTCRtpReceiver(this.dtlsTransport, 'video');
		//Edgeでは↑で動作するが、最新の仕様書では引数の順番が逆
		//this.audioReceiver = new RTCRtpReceiver('audio', this.dtlsTransport);
		//this.videoReceiver = new RTCRtpReceiver('video', this.dtlsTransport);

		//MediaStreamはイベントハンドラで取得するのではなく、自分で作る
		this.remoteStream = new MediaStream();
		this.remoteStream.addTrack(this.audioReceiver.track);
		this.remoteStream.addTrack(this.videoReceiver.track);
		//リスト6「ORTCでメディアを送受信する」（送信・受信）ここまで
		//========================================

		//対応しているファイル形式等の情報を交換
		this.localCaps = {
			send: {
				audio: RTCRtpSender.getCapabilities('audio'),
				video: RTCRtpSender.getCapabilities('video')
			},
			recv: {
				audio: RTCRtpReceiver.getCapabilities('audio'),
				video: RTCRtpReceiver.getCapabilities('video')
			}
		};
		//リスト5「Capabilitiesオブジェクト」(console.logで出力)
		console.log('get local capabilities');
		console.log(this.localCaps);
		socket.emit('capabilities', this.id, this.localCaps);
		this.checkCaps();
	}
	onRemoteParams(remoteParams){
		this.remoteParams = remoteParams;
		this.checkParams();
	}
	checkParams(){
		if(!this.localParams || !this.remoteParams){
			return;
		}
		this.iceTransport.start(this.iceGatherer, this.remoteParams.ice,
				this.isCaller?'controlling':'controlled');
		this.dtlsTransport.start(this.remoteParams.dtls);
	}
	onRemoteCaps(remoteCaps) {
		this.remoteCaps = remoteCaps;
		this.checkCaps();
	}
	checkCaps(){
		if(!this.localCaps || !this.remoteCaps){
			return;
		}
		this.audioSendParams = util.myCapsToSendParams(this.localCaps.send.audio, this.remoteCaps.recv.audio);
		this.videoSendParams = util.myCapsToSendParams(this.localCaps.send.video, this.remoteCaps.recv.video);
		this.audioRecvParams = util.myCapsToRecvParams(this.localCaps.recv.audio, this.remoteCaps.send.audio);
		this.videoRecvParams = util.myCapsToRecvParams(this.localCaps.recv.video, this.remoteCaps.send.video);

		this.audioRecvParams.muxId = null;
		this.audioRecvParams.encodings.push(util.RTCRtpEncodingParameters(1001, 0, 0, 0, 1.0));
		this.audioReceiver.receive(this.audioRecvParams);
		this.videoRecvParams.muxId = null;
		console.log(this.videoRecvParams.encodings);
		this.videoRecvParams.encodings.push(util.RTCRtpEncodingParameters(3003, 0, 0, 0, 1.0));
		this.videoReceiver.receive(this.videoRecvParams);
		if(this.audioSender){
			this.audioSendParams.encodings.push(util.RTCRtpEncodingParameters(1001, 0, 0, 0, 1.0));
			this.audioSender.send(this.audioSendParams);
		}
		if(this.videoSender){
			console.log(this.videoSendParams.encodings);
			this.videoSendParams.encodings.push(util.RTCRtpEncodingParameters(3003, 0, 0, 0, 1.0));
			this.videoSender.send(this.videoSendParams);
		}

		var mediaElement = document.createElement('video');
		mediaElement.autoplay = true;
		mediaElement.srcObject = this.remoteStream;
		document.querySelector('#stream').appendChild(mediaElement);
	}
}

function ringOff(){
	var remoteStream = document.querySelector('#stream');
	remoteStream.removeChild(remoteStream.firstElementChild);
	peer = null;
	targetId = null;
}

document.querySelector('#ringOff').addEventListener('click', ()=>{
	socket.emit('ringoff', peer.id);
	ringOff();
});

//発信側
//==========
document.querySelector('#callingFrom').addEventListener('submit', (e) => {
	var targetId = document.querySelector('#calleeId').value;
	//既に他のユーザと話中なら接続を行わない
	if(peer != null){
		e.preventDefault();
		return false;
	}

	socket.emit('offer', targetId);
	peer = new RTCPeer(targetId, true);

	e.preventDefault();
	return false;
});

//着信側
//==========
socket.on('offer', (userId) => {
	console.log(userId);
	if(peer != null){
		return;
	}
	peer = new RTCPeer(userId, false);
});

socket.on('capabilities', (userId, capabilities) => {
	if(peer && userId == peer.id){
		console.log(capabilities);
		peer.onRemoteCaps(capabilities);
	}
});

socket.on('icecandidate', (userId, iceCandidate) => {
	if(peer && userId == peer.id){
		console.log(iceCandidate);
		peer.iceTransport.addRemoteCandidate(iceCandidate);
	}
});

socket.on('localparams', (userId, payload) => {
	if(peer && userId == peer.id){
		peer.onRemoteParams(payload);
	}
});

socket.on('ringoff', (userId) => {
	if(peer && userId == peer.id){
		ringOff();
	}
});